//
// = FILENAME
//    PRMNode.hh
//
// = AUTHOR(S)
//    Patric Jensfelt
//
/*----------------------------------------------------------------------*/
#define MAX_ARMS 15
#ifndef PRMNode_hh
#define PRMNode_hh

#include "AStarNode.h"
#include "Position.h"

#include <cmath>

class PRMNode : public Position,
                public Cure::AStarNode
{
public:

  /**
   * Call this function to add a new edge to a certain node
   *
   * @param node pointer to node to add edge to
   */
  void addNewEdge(PRMNode* node);

  /**
   * Call this function to remove an edge (if it exists) to a certain node
   * 
   * @param node pointer to node to delete the edge to
   */
  void deleteEdge(PRMNode* node);

  /**
   * This function should return a list of nodes connected to this one
   * and the cost to move there
   * 
   * @param l list of connections to neighboring nodes and their costs
   * @return true always
   */
  bool getConnections(std::list< std::pair<AStarNode*, double> > &l);

protected:
  /// List of edges
  std::list<PRMNode*> m_Edges;

  double guessCostTo(AStarNode *n);
};

inline double
PRMNode::guessCostTo(AStarNode *n)
{
  int k;
  double dis=0, dt[MAX_ARMS];
  for(k=0;k<n_arms;k++) {
    dt[k] = ((PRMNode*)n)->geta()[k]-angles[k];
    if(dt[k] < -180.0)
      dt[k] += 360.0;
    else if (dt[k] > 180.0)
      dt[k] -= 360.0;
    dis += dt[k]*dt[k];
  }
    dis = sqrt(dis);
  return dis;
}

#endif // PRMNode_hh
