#include "Planner.h"
#include "PRM.h"
#ifndef WIN32
#include <unistd.h>  
#endif
#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <math.h>

Planner::Planner() {};
Planner::Planner(int n_a, double lgths[], double w, unsigned int n_obs, double obstlx1[], double obstly1[], double obstlx2[], double obstly2[], double start[], double end[])
{
  n_arms = n_a;
  width = w;
  n_obstacles = n_obs;
  for(int k=0;k<n_arms;k++) {
    lengths[k] = lgths[k];
    printf("lengths[%d]=%lf\n", k, lengths[k]);
  }
  printf("\n");
  for(int k=0;k<n_obstacles;k++) {
    obstaclesx1[k] = obstlx1[k];
    obstaclesy1[k] = obstly1[k];
    obstaclesx2[k] = obstlx2[k];
    obstaclesy2[k] = obstly2[k];
    printf("obstacles: %lf %lf %lf %lf\n", obstaclesx1[k], obstaclesy1[k], obstaclesx2[k], obstaclesy2[k]);
  }
  for(int k=0;k<n_arms;k++) {
    starta[k] = start[k];
    enda[k] = end[k];
  }
  col_count =0;
  ncol_count = 0;
}

Planner::~Planner()
{}

std::vector <std::vector <std::pair <int, int> > >
Planner::main1(int argc, char **argv)
{
  int i, j, k;
  std::string worldModel = "TestModel";
  World* world = 0;
  world = new Planner(n_arms, lengths, width, n_obstacles, obstaclesx1, obstaclesy1, obstaclesx2, obstaclesy2, starta, enda);
  PRM prm(n_arms);

  std::list<Position> path;
  int numNodes = 3000;
  int numEdges = 10;
  double stepSize = 10;
  
  prm.buildRoadMap(*world, numNodes, numEdges, stepSize);
  // double starta[]={0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  // double enda[] = {180,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  bool foundPath = prm.findPath(*world, starta, enda, path);
  double *itera, c_x[MAX_ARMS+1], c_y[MAX_ARMS+1], angle;
  int count=0;
  std::vector <std::vector <std::pair <int, int> > > result;
  std::vector <std::pair <int, int> > result1;
  for(std::list<Position>::iterator iter = path.begin(); iter != path.end();iter++, count++) {
    itera = iter->geta();
    c_x[0]=150;
    c_y[0]=150;
    angle = itera[0];
    result1.clear();
    result1.push_back(std::make_pair(150,150));
    printf("\nThetas: /%lf", itera[0]);
    for(int i=1;i<=n_arms;i++) {
      printf("%lf ", itera[i]);
      c_x[i]=c_x[i-1] + lengths[i-1]*cos(angle*(M_PI/180));
      c_y[i]=c_y[i-1] + lengths[i-1]*sin(angle*(M_PI/180));
      result1.push_back(std::make_pair((int)c_x[i],(int)c_y[i]));
      angle += itera[i];
    }

    printf("\nCoordinates: \n");
    for(k=0;k<=n_arms;k++) {
      printf("%lf %lf; ", c_x[k], c_y[k]);
    }
    printf("\n");
    result.push_back(result1);
  }
  return result;
}

bool 
Planner::collidesWith(double ars[])
{
  double c_x[MAX_ARMS+1], c_y[MAX_ARMS+1];
  c_x[0]=0;
  c_y[0]=0;
  double angle=ars[0];
  for(int i=1;i<=n_arms;i++) {
    c_x[i]=c_x[i-1] + lengths[i-1]*cos(angle*(M_PI/180));
    c_y[i]=c_y[i-1] + lengths[i-1]*sin(angle*(M_PI/180));
    angle += ars[i];
  }
  for(int i=0;i<n_arms;i++) 
    {
      for(int j=0;j<n_obstacles;j++)
	{
	  double rx1 = c_x[i];
	  double rx2 = c_x[i+1];
	  double ry1 = c_y[i];
	  double ry2 = c_y[i+1];
	  double ox1 = obstaclesx1[j];
	  double ox2 = obstaclesx2[j];
	  double oy1 = obstaclesy1[j];
	  double oy2 = obstaclesy2[j];
	  double a1 = ry2-ry1;
	  double b1 = rx1-rx2;
	  double a2 = oy2-oy1;
	  double b2 = ox1-ox2;
	  double det = a1*b2 - a2*b1;
	  if (det)
	    {
	      double c1 = rx1*ry2-rx2*ry1;
	      double c2 = ox1*oy2-ox2*oy1;
	      double inx = (b2*c1-b1*c2)/det;
	      double iny = (a1*c2-a2*c1)/det;
	      if((ox2 <= inx && ox1 >= inx || ox1 <= inx && ox2 >= inx) &&
		 (oy2 <= iny && oy1 >= iny || oy1 <= iny && oy2 >= iny) &&
		 (rx2 <= inx && rx1 >= inx || rx1 <= inx && rx2 >= inx) &&
		 (ry2 <= iny && ry1 >= iny || ry1 <= iny && ry2 >= iny)) 
		{
		  return true;
		}
	    }
	}
    }
  return false;
} 

int maint()
{
  //double lgths[]={12,23,42,42,12,4,42,42,11,32,12};
  double lgths[]={16,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5};
  double obstlx1[]={5.0, 2.5,  50.0, 52.5, 1,0,76,24,24,56,33,54,65,32};
  double obstly1[]={2.5, 5.0,  52.5, 50.0,  2,1,0,53,23,33,13,53,54,75,23};
  double obstlx2[]={2.5, 50.0, 52.5, 5.0,  1,0,1,34,64,74,24,24,74,76,33};
  double obstly2[]={5.0, 52.5, 50.0, 2.5,0,1,23,53,23,35,64,23,46,34};
  double starta[]={0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  double enda[] = {180,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  Planner *plan = new Planner(10, lgths, .1, 4, obstlx1, obstly1, obstlx2, obstly2, starta, enda);
  plan->main1(0,NULL);
}
